%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Función MATLAB para simular la estación del robot IRB120 (TFG_Rodrigo
% Sancho), tras haber sido procesada la pieza por visión artificial.
% En función de si la pieza es correcta o no, se simula el movimiento del
% robot cogiendo y depositando la pieza, o se simula el movimiento de la
% pieza descartada por el Conveyor
% Datos de Entrada:
%   Variable DescartarPieza, que indica si la pieza analizada es correcta
% Operaciones Realizadas:
%   Simulación de seleccion y deposito de pieza por el robot
%   Simulación de descarte de pieza
% Datos de Salida:
%   Ninguno
%
% Creado por:
%   - Rodrigo Sancho García
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function EstacionMATLAB2(DescartarPieza)
% Declaración de variables utilizadas
deg= pi/180;
nPts= 1e3;
Estacion= 'Estacion_TFG_RodrigoSancho.slx';
HOME= [0,0,0,0,90,0,0,0,0,0]*deg;
% Importa el modelo de árbol de cuerpo rígido desde el Modelo SimscapeMultibody 
robot= importrobot(Estacion);
% Evalua del Workspace una serie de parámetros
P_Recogida= evalin('base','P_Recogida');
P_Entrega= evalin('base','P_Entrega');
PosicionFigura= evalin('base','PosicionFigura');

% Variable para cambiar la herramienta y objeto de trabajo.
copiar= @(orig, dest) eval(['! copy .\Visual\',orig,'.stl  ','.\Visual\',dest,'.stl']);
% Asigna la herramienta y la carga del robot
tool1.name='Body17';
load1.name='Body18';

% Asigna la posición de la Figura
Pieza1.base= [[PosicionFigura,0,215]*1e-3, [0,0,0]*deg];
assignin('base','Pieza1',Pieza1.base);

% Se crea el objeto rob, con el que se controla el movimiento del Robot
rob= Kin(robot, Estacion, load1.name);

% Posicionar el robot en HOME
x= zeros(nPts,1);y=zeros(nPts,1);z= zeros(nPts,1);rot=zeros(nPts,1);Ini=ones(nPts,1);
t= linspace(0,1,nPts)';
CoordenadaX=[t,x];CoordenadaY=[t,y];CoordenadaZ=[t,z];RotacionZ=[t,rot];Inicial=[t,Ini];
assignin('base','CoordenadaX',CoordenadaX);assignin('base','CoordenadaY',CoordenadaY);
assignin('base','CoordenadaZ',CoordenadaZ);assignin('base','RotacionZ',RotacionZ);
assignin('base','Inicial',Inicial);
sim('Estacion_TFG_RodrigoSancho','StopTime','1');

% Sentencia condicional para decidir si realiza una acción u otra, en
% función de la validez de la pieza
if DescartarPieza==0
    % Pieza Válida -> Robot coge la Pieza
    y= zeros(nPts,1);
    CoordenadaX=[t,x];CoordenadaY=[t,y];CoordenadaZ=[t,z];
    assignin('base','CoordenadaX',CoordenadaX);assignin('base','CoordenadaY',CoordenadaY);
    assignin('base','CoordenadaZ',CoordenadaZ);
    rob.MoveJ(P_Recogida);
    rob.Tool(tool1.name);
    tool1.tcp= [[0,0,60]*1e-3, [0,0,0]*deg];
    assignin('base','tool1',tool1.tcp);
    copiar('Figura','load1');
    copiar('null','Figura');
    rob.MoveAbsJ(HOME);
    Izda= [90,0,0,0,90,0,0,0,0,0]*deg;
    rob.MoveAbsJ(Izda);
    rob.MoveJ(P_Entrega);
    copiar('load1','wobj7');
    copiar('null','load1');
    rob.MoveAbsJ(HOME);
else
    % Pieza No Válida -> Pieza Se desplaza por el Conveyor hasta regresar
    % al lugar de origen
    % Secuencia de Avance
    x= zeros(nPts,1);y=linspace(0,0.06,nPts)';z= zeros(nPts,1);rot=zeros(nPts,1);Ini=ones(nPts,1);
    CoordenadaX=[t,x];CoordenadaY=[t,y];CoordenadaZ=[t,z];RotacionZ=[t,rot];Inicial=[t,Ini];
    assignin('base','CoordenadaX',CoordenadaX);assignin('base','CoordenadaY',CoordenadaY);
    assignin('base','CoordenadaZ',CoordenadaZ);assignin('base','RotacionZ',RotacionZ);
    assignin('base','Inicial',Inicial);
    sim('Estacion_TFG_RodrigoSancho');
    
    % Secuencia de Rotacion sobre un punto exterior a la Pieza
    Pieza1.base= [[PosicionFigura,-400,215]*1e-3, [0,0,0]*deg];
    x= zeros(nPts,1);y=zeros(nPts,1);z= zeros(nPts,1);rot=linspace(0,0.45,nPts)';
    CoordenadaX=[t,x];CoordenadaY=[t,y];CoordenadaZ=[t,z];RotacionZ=[t,rot];
    assignin('base','CoordenadaX',CoordenadaX);assignin('base','CoordenadaY',CoordenadaY);
    assignin('base','CoordenadaZ',CoordenadaZ);assignin('base','RotacionZ',RotacionZ);
    assignin('base','Pieza1',Pieza1.base);
    sim('Estacion_TFG_RodrigoSancho');
    
    % Secuencia de Retroceso hasta el lugar de origen
    Pieza1.base= [[PosicionFigura+690,-400,215]*1e-3, [0,0,0]*deg];
    x= zeros(nPts,1);y=linspace(0,-0.25,nPts)';z= zeros(nPts,1);rot=zeros(nPts,1);
    CoordenadaX=[t,x];CoordenadaY=[t,y];CoordenadaZ=[t,z];RotacionZ=[t,rot];
    assignin('base','CoordenadaX',CoordenadaX);assignin('base','CoordenadaY',CoordenadaY);
    assignin('base','CoordenadaZ',CoordenadaZ);assignin('base','RotacionZ',RotacionZ);
    assignin('base','Pieza1',Pieza1.base);
    sim('Estacion_TFG_RodrigoSancho');
 end
end