%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Script .m MATLAB para establecer la conexión entre MATLAB-RobotStudio.
% Comunicación OPC
% Se leen las variables y señales de RobotStudio desde este fichero, y
% posteriormente se escriben desde MATLAB a RobotStudio.
% 
% Las variables escritas se modifican desde App Designer
%
% Creado por:
%   - Rodrigo Sancho García
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Inicialización de la comunicación OPC
InformacionHost = opcserverinfo('localhost'); %Informacion del Host
ServidoresDisponibles = InformacionHost.ServerID'; %Servidores Disponibles
Cliente = opcda('localhost', 'ABB.IRC5.OPC.Server.DA'); %Crear un cliente
connect(Cliente) %Conectar el cliente creado
Variables = addgroup(Cliente);%Variables modificadas con la comunicación
% Si se desea ver las variables que tiene el servidor, con un cierto nombre
% VerItemsServidor = serveritems(Cliente, '*NombreVariable*');
assignin('base','VariablesRobotStudio',Variables);

% Señales y Variables RobotStudio
PosInicial_Cubo=additem(Variables,'DESKTOP-F0D612S_IRB_120_3kg_058m.RAPID.T_ROB1.Maniobra_IRB120.PosInicial_Cubo');
Pos_Cubo=additem(Variables,'DESKTOP-F0D612S_IRB_120_3kg_058m.RAPID.T_ROB1.Maniobra_IRB120.Pos_Cubo');
Color_Cubo=additem(Variables,'DESKTOP-F0D612S_IRB_120_3kg_058m.RAPID.T_ROB1.Maniobra_IRB120.Color_Cubo');
TamanoX=additem(Variables,'DESKTOP-F0D612S_IRB_120_3kg_058m.RAPID.T_ROB1.Maniobra_IRB120.TamañoX');
TamanoY=additem(Variables,'DESKTOP-F0D612S_IRB_120_3kg_058m.RAPID.T_ROB1.Maniobra_IRB120.TamañoY');
TamanoZ=additem(Variables,'DESKTOP-F0D612S_IRB_120_3kg_058m.RAPID.T_ROB1.Maniobra_IRB120.TamañoZ');
CrearCubo=additem(Variables,'DESKTOP-F0D612S_IRB_120_3kg_058m.IOSYSTEM.IOSIGNALS.CrearCubo');
CambiarColorCubo=additem(Variables,'DESKTOP-F0D612S_IRB_120_3kg_058m.IOSYSTEM.IOSIGNALS.CambiarColorCubo');
PiezaNoValida=additem(Variables,'DESKTOP-F0D612S_IRB_120_3kg_058m.IOSYSTEM.IOSIGNALS.PiezaIncorrecta');
SeleccionT1=additem(Variables,'DESKTOP-F0D612S_IRB_120_3kg_058m.RAPID.T_ROB1.Maniobra_IRB120.SeleccionAccion');
SeleccionT2=additem(Variables,'DESKTOP-F0D612S_IRB14000.RAPID.T_ROB_R.Maniobra_Yumi_BrazoDcho.SeleccionAccion');
SeleccionT3=additem(Variables,'DESKTOP-F0D612S_IRB14000.RAPID.T_ROB_L.Maniobra_Yumi_BrazoIzdo.SeleccionAccion');
EstadoProceso=additem(Variables,'DESKTOP-F0D612S_IRB_120_3kg_058m.RAPID.T_ROB1.Maniobra_IRB120.EstadoProceso');
EjeX=additem(Variables,'DESKTOP-F0D612S_IRB_120_3kg_058m.RAPID.T_ROB1.Maniobra_IRB120.EjeX');
AlturaFigura=additem(Variables,'DESKTOP-F0D612S_IRB_120_3kg_058m.RAPID.T_ROB1.Maniobra_IRB120.Altura');
PiezaAnalizada=additem(Variables,'DESKTOP-F0D612S_IRB_120_3kg_058m.IOSYSTEM.IOSIGNALS.PiezaAnalizada');
MontajeRealizado=additem(Variables,'DESKTOP-F0D612S_IRB_120_3kg_058m.RAPID.T_ROB1.Maniobra_IRB120.MontajeRealizado');
SeleccionFigRealizada=additem(Variables,'DESKTOP-F0D612S_IRB_120_3kg_058m.RAPID.T_ROB1.Maniobra_IRB120.SeleccionFiguraRealizada');

% ACCIONES SIMULACION%%%%%%%%%%%%
% LecturaCubo=read(Pos_Cubo);
% Datos=LecturaCubo.Value;

% Evalua los parámetros asignados por el Usuario desde la interfaz, para
% poder escribirlos posteriormente en Robotstudio y simular la pieza
% deseada
Color = evalin('base','Color');
ColorDeseado = evalin('base','ColorDeseado');
Size = evalin('base','Size');
SizeDeseado = evalin('base','SizeDeseado');
PosicionInicial = evalin('base','Posicion');
PosicionX = evalin('base','PosicionX');
SeleccionModo = evalin('base','SeleccionModo');

% Reset Señales
write(CrearCubo,0);write(CambiarColorCubo,0);write(PiezaNoValida,0);
write(SeleccionT1,0);write(SeleccionT2,0);write(SeleccionT3,0);
write(PiezaAnalizada,0);
write(MontajeRealizado,'FALSE');write(SeleccionFigRealizada,'FALSE');

% Escribe el modo en Robotstudio que ha sido seleccionado por el usuario
write(SeleccionT1,SeleccionModo);write(SeleccionT2,SeleccionModo);write(SeleccionT3,SeleccionModo);

% En el caso de que el modo sea la detección de la Figura
if SeleccionModo==2
    % Se escribe en RobotStudio las variables, para crear el cubo segun ha
    % seleccionado el usuario
    write(TamanoX,Size(1));write(TamanoY,Size(2));write(TamanoZ,Size(3));
    write(Color_Cubo,Color);write(PosInicial_Cubo,PosicionInicial);
    write(CrearCubo,1);write(CambiarColorCubo,1);
    
%     Contador=0;
%     
%     while Contador<50
%         LecturaCubo=read(Pos_Cubo);
%         length(LecturaCubo.Value);
%         %[x,y,z]=FiltroValores(LecturaCubo.Value)
%         pause(0.5)
%         Contador=Contador+1;
%     end
    
    % Función para cambiar el formato del Color definido
    % Por ejemplo: RobotStudio=[255000,0,0] y MATLAB=[1,0,0];
    [ColorFormato1]=SeleccionColor(Color);
    
    % Se ejecuta la simulación de la pieza en MATLAB y se consiguen los
    % parámetros obtenidos por visión artificial 
    [ColorDetectado,AreaDetectada,PosicionDetectada] = EstacionMATLAB(ColorFormato1,Size,PosicionX);
    
    
    % Conversión de variables en función del tamaño detectado
    if (AreaDetectada>0 & AreaDetectada<=30)
        SizeDetectado='Pequeño';
        Altura=100;
        PosicionDetectada = mapfun(PosicionDetectada,12,21,1490,1690);
        assignin('base','PosicionDetectada',PosicionDetectada-1590);
    elseif (AreaDetectada>=70)
        SizeDetectado='Grande';
        Altura=200;
        PosicionDetectada = mapfun(PosicionDetectada,13.1786,19.8772,1490,1590);
        assignin('base','PosicionDetectada',PosicionDetectada-1540);
    elseif (AreaDetectada<70 & AreaDetectada>30)
        SizeDetectado='Mediano';
        Altura=150;
        PosicionDetectada = mapfun(PosicionDetectada,14.3711,18.5,1490,1640);
        assignin('base','PosicionDetectada',PosicionDetectada-1565);
    else
        SizeDetectado='NULL';
        Altura=0;
        assignin('base','PosicionDetectada',0);
    end
    
    assignin('base','ColorDetectado',ColorDetectado);
    assignin('base','SizeDetectado',SizeDetectado);
    
    % Sentencias condicionales para determinar si el tamaño y el color
    % detectado son los correctos
    if length(ColorDetectado)==length(ColorDeseado)
        if ColorDetectado==ColorDeseado
            ColorCorrecto=true;
        else
            ColorCorrecto=false;
        end
    else
        ColorCorrecto=false;
    end
    
    SizeCorrecto=false;
    switch SizeDeseado
        case 'Pequeño'
            if AreaDetectada<=30
                SizeCorrecto=true;
            end
        case 'Mediano'
            if AreaDetectada>30 & AreaDetectada<70
                SizeCorrecto=true;
            end
        case 'Grande'
            if AreaDetectada>=70
                SizeCorrecto=true;
            end
    end
    
    assignin('base','ColorCorrecto',ColorCorrecto);
    assignin('base','SizeCorrecto',SizeCorrecto);
    
    % Escribe en RobotStudio la posición y la altura de la figura detectada
    write(EjeX,PosicionDetectada);
    write(AlturaFigura,Altura);
end



%Funcion Map de Arduino
function output = mapfun(value,fromLow,fromHigh,toLow,toHigh)
narginchk(5,5)
nargoutchk(0,1)
output = (value - fromLow) .* (toHigh - toLow) ./ (fromHigh - fromLow) + toLow;
end


% function [x,y,z] = FiltroValores(Datos)
% if length(Datos)==16
%     x=strcat(Datos(2),Datos(3),Datos(4),Datos(5));
%     y=strcat(Datos(7),Datos(8),Datos(9),Datos(10),Datos(11));
%     z=strcat(Datos(13),Datos(14),Datos(15));
% elseif length(Datos)==15
%     x=strcat(Datos(2),Datos(3),Datos(4),Datos(5));
%     y=strcat(Datos(7),Datos(8),Datos(9),Datos(10));
%     z=strcat(Datos(12),Datos(13),Datos(14));
% elseif length(Datos)==14
%     x=strcat(Datos(2),Datos(3),Datos(4),Datos(5));
%     y=strcat(Datos(7),Datos(8),Datos(9),Datos(10));
%     z=strcat(Datos(12),Datos(13));
% elseif length(Datos)==13
%     x=strcat(Datos(2),Datos(3),Datos(4),Datos(5));
%     y=strcat(Datos(7),Datos(8),Datos(9));
%     z=strcat(Datos(11),Datos(12));
% end
% end